"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const fetchCookie = require("fetch-cookie");
const fs_1 = require("fs");
const mediawiki2_1 = require("mediawiki2");
const node_fetch_1 = require("node-fetch");
const playwright_1 = require("playwright");
const loggers_1 = require("./loggers");
const twitter_1 = require("./twitter");
const logger = (0, loggers_1.getLogger)('wiki');
const baseUrl = 'https://wiki.biligame.com/idolypride';
class default_1 {
    constructor(lock) {
        this.login = (sessdata) => playwright_1.firefox.launch().then(browser => {
            const jar = this.bot.cookieJar;
            return browser.newPage().then(page => page.context().addCookies([{
                    name: 'SESSDATA',
                    value: sessdata,
                    domain: '.biligame.com',
                    path: '/',
                }])
                .then(() => page.route('**/*.{png,jpg,jpeg,gif}', route => route.abort()))
                .then(() => page.route('*://*.baidu.com/**', route => route.abort()))
                .then(() => page.goto(`${baseUrl}/index.php?curid=2`, { waitUntil: 'networkidle' }))
                .then(() => { logger.info('logging in via browser...'); return page.context().cookies(); })
                .then(cookies => {
                const uidIndex = cookies.findIndex(cookie => cookie.name === 'gamecenter_wiki_UserName');
                if (!uidIndex)
                    throw new Error('auth error');
                return Promise.all(cookies.map(({ name, value, domain, path }) => jar.setCookie(`${name}=${value}; Domain=${domain}; Path=${path}`, baseUrl))).then(() => cookies[uidIndex].value);
            })
                .then(uid => {
                logger.info(`finished logging in via browser, wiki username: ${uid}`);
                this.bot.fetch = fetchCookie(node_fetch_1.default, jar);
                return browser.close();
            })
                .catch((err) => browser.close().then(() => {
                logger.fatal(`error logging in via browser, error: ${err}`);
                process.exit(0);
            })));
        });
        this.fetchMedia = (url) => new Promise((resolve, reject) => {
            logger.info(`fetching ${url}`);
            const fetch = () => (0, axios_1.default)({
                method: 'get',
                url,
                responseType: 'arraybuffer',
                timeout: 150000,
            }).then(res => {
                if (res.status === 200) {
                    logger.info(`successfully fetched ${url}`);
                    resolve(res.data);
                }
                else {
                    logger.error(`failed to fetch ${url}: ${res.status}`);
                    reject();
                }
            }).catch(err => {
                logger.error(`failed to fetch ${url}: ${err instanceof Error ? err.message : err}`);
                logger.info(`trying to fetch ${url} again...`);
                fetch();
            });
            fetch();
        }).then(data => {
            var _a;
            return (([_, filename, ext]) => {
                if (ext) {
                    const mediaFileName = `${filename}.${ext}`;
                    (0, fs_1.writeFileSync)(mediaFileName, Buffer.from(data));
                    return mediaFileName;
                }
                logger.warn('unable to find MIME type of fetched media, failing this fetch');
                throw Error();
            })((_a = /([^\/]*)\?format=([a-z]+)&/.exec(url)) !== null && _a !== void 0 ? _a : /([^\/]*)\.([^?]+)/.exec(url));
        });
        this.uploadMediaItems = (tweet, fileNamePrefix, indexOffset = 0) => {
            const mediaItems = [];
            if (tweet.extended_entities) {
                tweet.extended_entities.media.forEach((media, index) => {
                    let url;
                    if (media.type === 'photo') {
                        url = media.media_url_https.replace(/\.([a-z]+)$/, '?format=$1') + '&name=orig';
                    }
                    else {
                        url = media.video_info.variants
                            .filter(variant => variant.bitrate !== undefined)
                            .sort((var1, var2) => var2.bitrate - var1.bitrate)
                            .map(variant => variant.url)[0];
                    }
                    const mediaPromise = this.fetchMedia(url)
                        .then(mediaFileName => {
                        const filename = `${fileNamePrefix}${indexOffset + index + 1}.${mediaFileName.split('.')[1]}`;
                        logger.info(`uploading ${url} as ${filename}...`);
                        return this.bot.simpleUpload({
                            file: mediaFileName,
                            filename,
                        }).then(() => filename);
                    });
                    mediaItems.push(mediaPromise);
                });
            }
            return Promise.all(mediaItems);
        };
        this.appendMedia = (tweet, genre, indexOffset) => {
            const { pageTitle } = (0, twitter_1.processTweetBody)(tweet);
            return this.uploadMediaItems(tweet, `公告-${genre}-${pageTitle}-`, indexOffset)
                .then(fileNames => {
                logger.info(`updating page 公告/${pageTitle}...`);
                return this.bot.edit({
                    title: `公告/${pageTitle}`,
                    appendtext: `${fileNames.map(fileName => `[[文件:${fileName}|无框|左]]\n`).join('')}`,
                    bot: true,
                    notminor: true,
                    nocreate: true,
                })
                    .then(({ new: isNewPost, newtimestamp, pageid, result, title }) => ({
                    pageid,
                    title,
                    new: isNewPost,
                    mediafiles: fileNames,
                    result,
                    timestamp: new Date(newtimestamp).toString(),
                }))
                    .catch(error => {
                    logger.error(`error updating page, error: ${error}`);
                    return {
                        pageid: undefined,
                        title: `公告/${pageTitle}`,
                        new: undefined,
                        mediafiles: [],
                        result: 'Failed',
                        timestamp: undefined,
                    };
                });
            });
        };
        this.post = (tweet, genre) => {
            const { title, body, pageTitle, date } = (0, twitter_1.processTweetBody)(tweet);
            const sameTitleAction = this.lock.lastActions.find(action => action.title === title);
            if (sameTitleAction)
                return this.appendMedia(tweet, genre, sameTitleAction.mediafiles.length);
            return this.uploadMediaItems(tweet, `公告-${genre}-${pageTitle}-`)
                .then(fileNames => {
                logger.info(`creating page 公告/${pageTitle}...`);
                return this.bot.edit({
                    title: `公告/${pageTitle}`,
                    basetimestamp: new Date(),
                    text: `{{文章戳
|文章上级页面=公告
|子类别=${genre}
|时间=${date}
|作者=IDOLY PRIDE
|是否原创=否
|来源=[https://twitter.com/idolypride IDOLY PRIDE]
|原文地址=[https://twitter.com/idolypride/status/${tweet.id_str} ${pageTitle}]
}}
====${title}====
<poem>
${body}
</poem>
${fileNames.map(fileName => `[[文件:${fileName}|无框|左]]`).join('\n')}
`,
                    bot: true,
                    notminor: true,
                    createonly: true,
                })
                    .then(({ new: isNewPost, newtimestamp, pageid, result, title }) => ({
                    pageid,
                    title,
                    new: isNewPost,
                    mediafiles: fileNames,
                    result,
                    timestamp: new Date(newtimestamp).toString(),
                }))
                    .catch(error => {
                    logger.error(`error creating page, error: ${error}`);
                    return {
                        pageid: undefined,
                        title: `公告/${pageTitle}`,
                        new: undefined,
                        mediafiles: [],
                        result: 'Failed',
                        timestamp: undefined,
                    };
                });
            });
        };
        this.bot = new mediawiki2_1.MWBot(`${baseUrl}/api.php`);
        this.lock = lock;
    }
}
exports.default = default_1;
